@@ -40,11 +40,22 @@ Util::H2O - Hash to Object: turns hashrefs into objects with accessors for keys
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.06';
 # For AUTHOR, COPYRIGHT, AND LICENSE see the bottom of this file
 
 our @EXPORT = qw/ h2o /;  ## no critic (ProhibitAutomaticExportation)
 
+BEGIN {
+	# lock_ref_keys wasn't available until Hash::Util 0.06 / Perl v5.8.9
+	# (note the following will probably also fail on the Perl v5.9 dev releases)
+	# uncoverable branch false
+	# uncoverable condition false
+	if ( $] ge '5.008009' ) {
+		require Hash::Util;
+		Hash::Util->import('lock_ref_keys') }
+	else { *lock_ref_keys = sub {} }  # uncoverable statement
+}
+
 =head1 Description
 
 This module allows you to turn hashrefs into objects, so that instead
@@ -52,6 +63,11 @@ of C<< $hash->{key} >> you can write C<< $hash->key >>, plus you get
 protection from typos. In addition, options are provided that allow
 you to whip up really simple classes.
 
+You can still use the hash like a normal hashref as well, as in
+C<< $hash->{key} >>, C<keys %$hash>, and so on, but note that by
+default this function also locks the hash's keyset to prevent typos
+there too.
+
 This module exports a single function by default.
 
 =head2 C<h2o I<@opts>, I<$hashref>, I<@additional_keys>>
@@ -67,7 +83,7 @@ one will take effect.
 
 Nested hashes are objectified as well. Note that I<none> of the other
 options will be applied to the nested hashes, including
-C<@additional_keys>.
+C<@additional_keys>. Nested arrayrefs are not recursed into.
 
 =item C<-meth>
 
@@ -91,8 +107,9 @@ C<-new>.
 Generates a constructor named C<new> in the package. The constructor
 works as a class and instance method, and dies if it is given any
 arguments that it doesn't know about. If you want more advanced
-features, like required arguments or other validation, you should
-probably switch to something like L<Moo> instead.
+features, like required arguments, validation, or other
+initialization, you should probably switch to something like L<Moo>
+instead.
 
 =item C<< -clean => I<bool> >>
 
@@ -101,12 +118,31 @@ destroyed. Defaults to I<false> when C<-class> is specified, I<true>
 otherwise. If this is I<false>, be aware that the packages will stay
 in Perl's symbol table and use memory accordingly.
 
+=item C<< -lock => I<bool> >>
+
+Whether or not to use L<Hash::Util>'s C<lock_ref_keys> to prevent
+modifications to the hash's keyset. Defaults to I<true> (versions of
+this module before v0.06 did not lock the keyset).
+
+Keysets of objects created by the constructor generated by the
+C<-new> option are I<not> locked.
+
+Note that on really old Perls, that is, before Perl v5.8.9,
+L<Hash::Util> and its C<lock_ref_keys> are not available, so the hash
+is never locked on those versions of Perl.
+
 =back
 
 =head3 C<$hashref>
 
-You must supply a plain (unblessed) hash reference here. Be aware that
-this function I<does> modify the original hashref(s) by blessing it.
+You must supply a plain (unblessed) hash reference here. Be aware
+that this function I<does> modify the original hashref(s) by blessing
+it and locking its keyset (the latter can be disabled with the
+C<-lock> option).
+
+An accessor will be set up for each key in the hash; note that the
+keys must of course be valid Perl identifiers for you to be able to
+call the method normally.
 
 When C<-clean> is I<true> (the default, unless you use C<-class>),
 the hash may not contain a key named C<DESTROY>. When C<-new> is
@@ -123,11 +159,12 @@ The (now blessed) C<$hashref>.
 =cut
 
 sub h2o {  ## no critic (RequireArgUnpacking, ProhibitExcessComplexity)
-	my ($recurse,$meth,$class,$new,$clean);
+	my ($recurse,$meth,$class,$new,$clean,$lock);
 	while ( @_ && $_[0] && !ref$_[0] ) {
 		if ($_[0] eq '-recurse' ) { $recurse = shift }  ## no critic (ProhibitCascadingIfElse)
 		elsif ($_[0] eq '-meth' ) { $meth    = shift }
 		elsif ($_[0] eq '-clean') { $clean   = (shift, shift()?1:0) }
+		elsif ($_[0] eq '-lock' ) { $lock    = (shift, shift()?1:0) }
 		elsif ($_[0] eq '-new'  ) { $new     = shift }
 		elsif ($_[0] eq '-class') {
 			$class = (shift, shift);
@@ -137,6 +174,7 @@ sub h2o {  ## no critic (RequireArgUnpacking, ProhibitExcessComplexity)
 		else { croak "unknown option to h2o: '$_[0]'" }
 	}
 	$clean = !defined $class unless defined $clean;
+	$lock = 1 unless defined $lock;
 	my $hash = shift;
 	croak "h2o must be given a plain hashref" unless ref $hash eq 'HASH';
 	my %keys = map {$_=>1} @_, keys %$hash;
@@ -166,7 +204,9 @@ sub h2o {  ## no critic (RequireArgUnpacking, ProhibitExcessComplexity)
 		};
 		{ no strict 'refs'; *{$pack.'::new'} = $sub }  ## no critic (ProhibitNoStrict)
 	}
-	return bless $hash, $pack;
+	bless $hash, $pack;
+	lock_ref_keys $hash, keys %keys if $lock;
+	return $hash;
 }
 
 1;
